/***************************************************************************
                          audiodlg.cpp  -  description
                             -------------------
    begin                : Fri May 16 2003
    copyright            : (C) 2003 by Reinier Lamers
    email                : tux_rocker@planet.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// 16-08-2005: Added JACK support - Reinier Lamers
// 22-08-2003: Added ALSA support, and included config.h and added #ifdefs for audio system selection at compile time - Reinier Lamers

#include <config.h>
#include <kdialogbase.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <klocale.h>
#include "audiodlg.h"
#include "guitune.h"

AudioDialog::AudioDialog( MainWidget *mnwdgt, QWidget *parent, const char *name )
  : KDialogBase( parent, name, true, i18n("Audio Settings"), Ok|Cancel )
{
  mainwidget = mnwdgt;
  
  QVBox *vbox = makeVBoxMainWidget();
  QButtonGroup *group = new QButtonGroup( 2, Horizontal, i18n("Audio System"), vbox );
  group->setExclusive( true );
#ifdef USE_ARTS
  QCheckBox *arts = new QCheckBox( "aRts", group );
  connect( arts, SIGNAL(toggled(bool)), this, SLOT(setSoundInputTypeaRts(bool)) );
#endif
#ifdef USE_OSS
  QCheckBox *oss = new QCheckBox( "OSS", group );
  connect( oss, SIGNAL(toggled(bool)), this, SLOT(setSoundInputTypeOSS(bool)) );
#endif
#ifdef USE_ALSA
  QCheckBox *alsa = new QCheckBox( "ALSA", group );
  connect( alsa, SIGNAL(toggled(bool)), this, SLOT(setSoundInputTypeALSA(bool)) );
#endif
#ifdef USE_JACK
  QCheckBox *jack = new QCheckBox( "JACK", group );
  connect( jack, SIGNAL(toggled(bool)), this, SLOT(setSoundInputTypeJACK(bool)) );
#endif
  QCheckBox *autodetect = new QCheckBox( i18n("Autodetect"), group );
  connect( autodetect, SIGNAL(toggled(bool)), this, SLOT(setSoundInputTypeAutodetect(bool)) );

  switch (mnwdgt->getSoundInputType())
  {
#ifdef USE_ARTS
  case MainWidget::aRts:
    arts->toggle();
    break;
#endif
#ifdef USE_OSS
  case MainWidget::OSS:
    oss->toggle();
    break;
#endif
#ifdef USE_ALSA
  case MainWidget::ALSA:
    alsa->toggle();
    break;
#endif
#ifdef USE_JACK
  case MainWidget::JACK:
    jack->toggle();
    break;
#endif
  case MainWidget::autodetect:
    autodetect->toggle();
    break;
  }

#ifdef USE_OSS
  QHBox *dsphbox = new QHBox( vbox );
  QLabel *dsplabel = new QLabel( i18n("OSS Device:"), dsphbox );  
  QLineEdit *dsp_lineedit = new QLineEdit( dsphbox );
  connect( dsp_lineedit, SIGNAL(textChanged(const QString &)), this, SLOT(setDSPName(const QString &)) );
  dsp_lineedit->setText( mnwdgt->getDSPName() );
#endif
#ifdef USE_ALSA
  QHBox *alsahbox = new QHBox( vbox );
  QLabel *alsalabel = new QLabel( i18n("ALSA Device:"), alsahbox );
  QLineEdit *alsa_lineedit = new QLineEdit( alsahbox );
  connect( alsa_lineedit, SIGNAL(textChanged(const QString &)), this, SLOT(setALSAName(const QString &)) );
  alsa_lineedit->setText( mnwdgt->getALSAName() );
#endif
}

AudioDialog::~AudioDialog()
{
}

void AudioDialog::slotOk( void )
{
  if (soundinputtype != mainwidget->getSoundInputType())
    mainwidget->setSoundInputType( soundinputtype );
#ifdef USE_OSS
  if ( strcmp(dsp_devicename, mainwidget->getDSPName()) )
    mainwidget->setDSPName( dsp_devicename );
#endif
#ifdef USE_ALSA
  if ( strcmp(alsa_devicename, mainwidget->getALSAName()) )
    mainwidget->setALSAName( alsa_devicename );
#endif
  accept();  
}

/* Same as in the header, we can't #ifdef slots away because moc ignores preproc statements */
void AudioDialog::setSoundInputTypeaRts( bool on )
{
#ifdef USE_ARTS
  if (on)
  {
    soundinputtype = MainWidget::aRts;
  }
#endif
}

void AudioDialog::setSoundInputTypeOSS( bool on )
{
#ifdef USE_OSS
  if (on)
  {
    soundinputtype = MainWidget::OSS;
  }
#endif
}

void AudioDialog::setSoundInputTypeALSA( bool on )
{
#ifdef USE_ALSA
  if (on) 
  {
    soundinputtype = MainWidget::ALSA;
  }
#endif
}

void AudioDialog::setSoundInputTypeJACK( bool on )
{
#ifdef USE_JACK
  if (on)
  {
    soundinputtype = MainWidget::JACK;
  }
#endif
}


void AudioDialog::setSoundInputTypeAutodetect( bool on )
{
  if (on)
  {
    soundinputtype = MainWidget::autodetect;
  }
}

void AudioDialog::setDSPName( const QString &name )
{
#ifdef USE_OSS
  strncpy(dsp_devicename, name.latin1(), sizeof(dsp_devicename)-1);
  dsp_devicename[sizeof(dsp_devicename)-1] = '\0';
#endif
}

void AudioDialog::setALSAName( const QString &name )
{
#ifdef USE_ALSA
  strncpy( alsa_devicename, name.latin1(), sizeof(alsa_devicename) - 1 );
  alsa_devicename[sizeof(alsa_devicename)-1] = '\0';
#endif
}
