//
//
//    guitune - program for tuning instruments (actually an oscilloscope)
//    Copyright (C) 1999  Florian Berger
//    Email: florian.berger@jk.uni-linz.ac.at
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License Version 2 as
//    published by the Free Software Foundation;
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//

// 22-08-2003: Added ALSA support, and included config.h and added #ifdefs for audio system selection at compile time - Reinier Lamers
// 29-4-2003: moved to new sound input system, added getDSPName(), getSoundInputType(), setSoundInputType() etc.
// Modified 2002 by Reinier Lamers

#ifndef GUITUNE_H
#define GUITUNE_H

#include <config.h>
#include <qwidget.h>
#include <qcheckbox.h>


class    OsziView;
class    LogView;
class    QLCDNumber;
class    QTimer;
class    QMenuBar;
class    QPopupMenu;
class    QSpinBox;
class    SoundInput;

class MainWidget : public QWidget
{
 Q_OBJECT
 public:
   /* use fixed numbers in the enum so that configuration files will be portable between differently configured builds */
   enum SoundInputType {
     autodetect = 0
#ifdef USE_OSS
     ,OSS = 1
#endif
#ifdef USE_ALSA
     ,ALSA = 2
#endif
#ifdef USE_ARTS
     ,aRts = 3
#endif
#ifdef USE_JACK
     ,JACK = 4
#endif
   };
   MainWidget( QWidget *parent, const char *name , int argc, char **argv);
   ~MainWidget();
   int sampnr;
   int sampfreq;
   double sampfreq_exact;
   int blksize;
   int trig1;
   int trig2;
   int    note_0t;
   int    note_ht;
   double freq_0t;
   double freq_ht;
   double lfreq_0t;
   double lfreq_ht;
   void showOszi();
   void hideOszi();
   void showLogView();
   void hideLogView();
   bool getNotch();
   char *getDSPName( void );
   char *getALSAName( void );
   SoundInputType getSoundInputType( void );
   void setSoundInputType( SoundInputType _soundinputtype );
   void start_audio( void );
 public slots:
   void setSampFreq(int f);
   void setSampNr(int nr);
   void setNotch();
   /* ifdeffing out slots is impossible, because of moc */
   void setDSPName(const char* name);
   void setALSAName( const char *name );
   void setTuningNorm();
   void setTuningWien();
   void setTuningPhys();
   void setTuningEqui();
   void setTuningNat();
   void setScaleUS();
   void setScaleUSAlt();
   void setScaleGE();
   void setScaleGEAlt();
   void setScaleFR();
   void setScaleFRAlt();
   void showAudioDialog( void );
 signals:
   void signalSampFreqChanged();
   void signalSampNrChanged();
   void signalNotchChanged();

 protected:
   void paintEvent(QPaintEvent *);
   void showEvent(QShowEvent *);
   void resizeEvent(QResizeEvent *);
//   void keyPressEvent(QKeyEvent *);
//   void keyReleaseEvent(QKeyEvent *);
 private:
   SoundInput *soundinput;
   SoundInputType soundinputtype;
   OsziView   *oszi;
   LogView    *logview;
   QLCDNumber *freqview;
   QLCDNumber *nfreqview;
   QTimer     *timer;
   QMenuBar   *menu;
   QPopupMenu *filemenu;
   QPopupMenu *optmenu;
   QPopupMenu *viewmenu;
   QSpinBox   *sampfreq_input;
   QSpinBox   *sampnr_input;
   QCheckBox  *notch_input;
   /* this boolean indicates if soundinput must be recreated before using it
    * (because of a change in the settings) */
   bool must_restart_audio;
   int oszi_height;
   unsigned char sample[64000];
   double freqs[12];
   double lfreqs[12];
   int    processing_audio;

   /* Leave these two in even if no OSS and/or ALSA is configured, because SoundInput::autodetect wants them as argument */
   char dsp_devicename[100];
   char alsa_devicename[100];

   void stop_audio( void );
 private slots:
   void proc_audio();
};

#endif /* GUITUNE_H */
