/***************************************************************************
                          k2guitune.cpp  -  description
                             -------------------
    begin                : Sun Aug  5 15:37:33 CEST 2001
    copyright            : (C) 2001 by Florian Berger
    email                : harpin_floh@yahoo.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// 22-08-2003: added ALSA support and #ifdef's (and #include <config.h>) for compile time audio system selection - Reinier Lamers
// Modified 2003 by Reinier Lamers: added more configuration possibilities in readOptions() and saveOptions()
// Modified 2002 by Reinier Lamers

#include <config.h>

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <klocale.h>

// application specific includes
#include "k2guitune.h"
#include "guitune.h"

#define ID_STATUS_MSG 1

K2GuituneApp::K2GuituneApp(QWidget* , const char* name):KMainWindow(0, name)
{
  config=kapp->config();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initView();
  initActions();

  readOptions();
}

K2GuituneApp::~K2GuituneApp()
{
}

void K2GuituneApp::initActions()
{
  // file menu
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection(), "file_quit");
  fileQuit->setStatusText(i18n("Quits the application"));

  // settings menu
  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection(), "view_toolbar");
  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection(), "view_statusbar");
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));
  viewOszi = new KToggleAction(i18n("&Oscilloscope"), 0, this, SLOT(m_view_oszi()), actionCollection(), "view_oszi");
  viewOszi->setStatusText(i18n("Enables/disables the oscilloscope"));
  viewOszi->setChecked( true );
  viewLogView = new KToggleAction(i18n("&Logview"), 0, this, SLOT(m_view_logview()), actionCollection(), "view_logview");
  viewLogView->setStatusText(i18n("Enables/disables the logview"));
  viewLogView->setChecked( true );

  //settings menu
  //settings menu -> tuning menu
  tuningNormal = new KRadioAction(i18n("&Normal"), 0, view, SLOT(setTuningNorm()), actionCollection(), "tuning_norm");
  tuningNormal->setExclusiveGroup("tuning_pitch");
  tuningNormal->setChecked( true );
  tuningWien = new KRadioAction(i18n("&Wien"), 0, view, SLOT(setTuningWien()), actionCollection(), "tuning_wien");
  tuningWien->setExclusiveGroup("tuning_pitch");
  tuningPhys = new KRadioAction(i18n("&Physical"), 0, view, SLOT(setTuningPhys()), actionCollection(), "tuning_phys");
  tuningPhys->setExclusiveGroup("tuning_pitch");

  tuningNat = new KRadioAction(i18n("N&atural"), 0, view, SLOT(setTuningNat()), actionCollection(), "tuning_nat");
  tuningNat->setExclusiveGroup("tuning_mode");
  tuningEqui = new KRadioAction(i18n("&Equidistant"), 0, view, SLOT(setTuningEqui()), actionCollection(), "tuning_equi");
  tuningEqui->setExclusiveGroup("tuning_mode");
  tuningEqui->setChecked( true );

  //settings menu -> scale menu
  scaleUS = new KRadioAction(i18n("&US"), 0, view, SLOT(setScaleUS()), actionCollection(), "scale_us");
  scaleUS->setExclusiveGroup("scale");
  scaleUS->setChecked( true );
  scaleUSAlt = new KRadioAction(i18n("U&S-Alternate"), 0, view, SLOT(setScaleUSAlt()), actionCollection(), "scale_us_alt");
  scaleUSAlt->setExclusiveGroup("scale");
  scaleGerm = new KRadioAction(i18n("&German"), 0, view, SLOT(setScaleGE()), actionCollection(), "scale_german");
  scaleGerm->setExclusiveGroup("scale");
  scaleGermAlt =  new KRadioAction(i18n("G&erman-Alternate"), 0, view, SLOT(setScaleGEAlt()), actionCollection(), "scale_german_alt");
  scaleGermAlt->setExclusiveGroup("scale");
  scaleFrench = new KRadioAction(i18n("&French"), 0, view, SLOT(setScaleFR()), actionCollection(), "scale_french");
  scaleFrench->setExclusiveGroup("scale");
  scaleFrenchAlt =  new KRadioAction(i18n("F&rench-Alternate"), 0, view, SLOT(setScaleFRAlt()), actionCollection(), "scale_french_alt");
  scaleFrenchAlt->setExclusiveGroup("scale");

  //settings menu toplevel
  audioSettings = new KAction(i18n("&Audio Settings..."), 0, view, SLOT(showAudioDialog()), actionCollection(), "audio_settings");

  // use the absolute path to your k2guituneui.rc file for testing purpose in createGUI();
  createGUI();
}


void K2GuituneApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
}

void K2GuituneApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new MainWidget(this,"MainWidget",0,(char **)0);
//  view = new K2GuituneView(this);
  setCentralWidget(view);
//  setCaption(doc->URL().fileName(),false);

}

void K2GuituneApp::saveOptions()
{
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", viewToolBar->isChecked());
  config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());

  config->setGroup("Audio Options");
  config->writeEntry("Input Method", (int)view->getSoundInputType());
  #ifdef USE_OSS
  config->writeEntry("DSP Device", view->getDSPName() );
  #endif
  #ifdef USE_ALSA
  config->writeEntry("ALSA Device", view->getALSAName() );
  #endif
//  fileOpenRecent->saveEntries(config,"Recent Files");
}


void K2GuituneApp::readOptions()
{

  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();


  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }

  config->setGroup("Audio Options");
  #ifdef USE_OSS
  view->setDSPName( config->readPathEntry("DSP Device", "/dev/dsp") );
  #endif
  #ifdef USE_ALSA
  view->setALSAName( config->readPathEntry("ALSA Device", "default") );
  #endif
  view->setSoundInputType( (MainWidget::SoundInputType)config->readNumEntry("Input Method", (int)MainWidget::autodetect) );

  /* now that all settings have been done, we're ready to go */
  view->start_audio();
}

void K2GuituneApp::saveProperties(KConfig *_cfg)
{
}


void K2GuituneApp::readProperties(KConfig* _cfg)
{
}

bool K2GuituneApp::queryClose()
{
}

bool K2GuituneApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

/*
 * Originally this function never reached its end (because of KDE2/KDE3 different behavior?).
 * Just simplified it a bit :)
 */
void K2GuituneApp::slotFileQuit()
{
  kapp->exit();
}

void K2GuituneApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }

  slotStatusMsg(i18n("Ready."));
}

void K2GuituneApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}


void K2GuituneApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

void K2GuituneApp::m_view_oszi()
{
  if(!viewOszi->isChecked())
  {
    view->hideOszi();
  } else {
    view->showOszi();
  }
}

void K2GuituneApp::m_view_logview()
{
  if (!viewLogView->isChecked())
  {
    view->hideLogView();
  } else {
    view->showLogView();
  }
}
