/***************************************************************************
                          k2guitune.h  -  description
                             -------------------
    begin                : Sun Aug  5 15:37:33 CEST 2001
    copyright            : (C) 2001 by Florian Berger
    email                : harpin_floh@yahoo.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Modified 2002 by Reinier Lamers

#ifndef K2GUITUNE_H
#define K2GUITUNE_H


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt

// include files for KDE
#include <kapp.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>

// forward declaration of the K2Guitune classes
//class K2GuituneDoc;
//class K2GuituneView;
class MainWidget;  // Guitune class

/**
  * The base class for K2Guitune application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of K2GuituneView creates your center view, which is connected
  * to the window's Doc object.
  * K2GuituneApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as using KActions.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 1.2 code generation
  */
class K2GuituneApp : public KMainWindow
{
  Q_OBJECT

//  friend class K2GuituneView;
  friend class MainWidget;

  public:
    /** construtor of K2GuituneApp, calls all init functions to create the application.
     */
    K2GuituneApp(QWidget* parent=0, const char* name=0);
    ~K2GuituneApp();

  protected:
    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */
    void saveOptions();
    /** read general Options again and initialize all variables like the recent file list
     */
    void readOptions();
    /** initializes the KActions of the application */
    void initActions();
    /** sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initStatusBar();
    /** initializes the document object of the main window that is connected to the view in initView().
     * @see initView();
     */
    void initView();
    /** queryClose is called by KTMainWindow on each closeEvent of a window. Against the
     * default implementation (only returns true), this calles saveModified() on the document object to ask if the document shall
     * be saved if Modified; on cancel the closeEvent is rejected.
     * @see KTMainWindow#queryClose
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryClose();
    /** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's
     * properties.
     * @see KTMainWindow#queryExit
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryExit();
    /** saves the window properties for each open window during session end to the session config file, including saving the currently
     * opened file by a temporary filename provided by KApplication.
     * @see KTMainWindow#saveProperties
     */
    virtual void saveProperties(KConfig *_cfg);
    /** reads the session config file and restores the application's state including the last opened files and documents by reading the
     * temporary files saved by saveProperties()
     * @see KTMainWindow#readProperties
     */
    virtual void readProperties(KConfig *_cfg);

  public slots:
    void slotFileQuit();
    /** toggles the toolbar
     */
    void slotViewToolBar();
    /** toggles the statusbar
     */
    void slotViewStatusBar();
    /** changes the statusbar contents for the standard label permanently, used to indicate current actions.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text);

  private:
    /** the configuration object of the application */
    KConfig *config;
    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    MainWidget *view;  //Guitune widget class
//    K2GuituneView *view;

    // KAction pointers to enable/disable actions
    KAction *fileQuit;
    KToggleAction *viewToolBar;
    KToggleAction *viewStatusBar;
    KToggleAction *viewOszi;
    KToggleAction *viewLogView;
    KRadioAction *tuningNormal;
    KRadioAction *tuningWien;
    KRadioAction *tuningPhys;
    KRadioAction *tuningNat;
    KRadioAction *tuningEqui;
    KRadioAction *scaleUS;
    KRadioAction *scaleUSAlt;
    KRadioAction *scaleGerm;
    KRadioAction *scaleGermAlt;
    KRadioAction *scaleFrench;
    KRadioAction *scaleFrenchAlt;
    KAction *audioSettings;


//    QPopupMenu *filemenu;

  private slots:
    void m_view_oszi();
    void m_view_logview();
};

#endif // K2GUITUNE_H
